package rychlosti;

import java.util.*;

class TypyTestu {
  public static final int POCET_RADU = 2;
  public static final boolean DEBUG = false;   // pro ladeni
  public static int pocetPrvku = 100;
  public static Integer[] poleVzoru = null;

  private List<Integer> li;
  private final String typListu;

  TypyTestu(List<Integer> li) {
    this.li = li;
    this.typListu = li.getClass().getSimpleName();
  }

  private void tisk() {
    if (DEBUG == true) {
      System.out.print(li + " ");
    }
  }

  private static abstract class Akce {
    String typ;

    Akce(String typ) {
      this.typ = typ;
    }

    abstract void akce();
  }

  private final Akce[] poleAkci = {
    new Akce("naplneni po jednom") {
      void akce() {
        for (int i = 0;  i < pocetPrvku;  i++) {
          li.add(poleVzoru[i]);
        }
        tisk();
      }
    },

    new Akce("pruchodIndexaci") {
      void akce() {
        int vel = li.size();
        for (int i = 0;  i < vel;  i++) {
          li.get(poleVzoru[i]);
        }
      }
    },

    new Akce("pruchodIteratorem") {
      void akce() {
        for (Iterator<Integer> it = li.iterator();  it.hasNext(); ) {
          it.next();
        }
      }
    },

    new Akce("vypusteniPolovinyIndexaciZezadu") {
      void akce() {
        int vel = li.size();
        for (int i = vel - 2;  i >= 0;  i -= 2) {
          li.remove(poleVzoru[i]);
        }
        tisk();
      }
    },

    new Akce("vlozeniPolovinyIndexaci") {
      void akce() {
        int vel = li.size() * 2;
        for (int i = 0;  i < vel;  i += 2) {
          li.add(i, poleVzoru[i]);
        }
        tisk();
      }
    },

    new Akce("vypusteniPolovinyIndexaciZepredu") {
      void akce() {
        for (int i = 0;  i < li.size();  i++) {
          li.remove(i);
        }
        tisk();
      }
    },

    new Akce("clearANaplneni") {
      void akce() {
        li.clear();
        for (int i = 0;  i < pocetPrvku;  i++) {
          li.add(poleVzoru[i]);
        }
        tisk();
      }
    },

    new Akce("vypusteniPolovinyIteratorem") {
      void akce() {
        for (Iterator<Integer> it = li.iterator();  it.hasNext(); ) {
          it.next();
          it.remove();
          it.next();           // spinavy trik - nepouzita hasNext()
        }
        tisk();
      }
    },

    new Akce("clearANaplneni") {
      void akce() {
        li.clear();
        for (int i = 0;  i < pocetPrvku;  i++) {
          li.add(poleVzoru[i]);
        }
        tisk();
      }
    },

    new Akce("serazeniSestupne") {
      void akce() {
        Collections.sort(li, Collections.reverseOrder());
        tisk();
      }
    },

    new Akce("serazeniVzestupne") {
      void akce() {
        Collections.sort(li);
        tisk();
      }
    },

    new Akce("zamichani") {
      void akce() {
        Collections.shuffle(li);
        tisk();
      }
    },

    new Akce("serazeniVzestupne") {
      void akce() {
        Collections.sort(li);
        tisk();
      }
    },

    new Akce("binarniHledaniOdzadu") {
      void akce() {
        int vel = li.size() - 1;
        for (int i = 0;  i <= vel;  i++) {
          Collections.binarySearch(li, poleVzoru[vel - i]);
        }
      }
    },

    new Akce("otoceniPoradi") {
      void akce() {
        Collections.reverse(li);
        tisk();
      }
    },

    new Akce("hledaniOdzadu") {
      void akce() {
        int vel = li.size();
        for (int i = 0;  i < vel;  i++) {
          li.indexOf(poleVzoru[i]);
        }
      }
    },

    new Akce("zamichani") {
      void akce() {
        Collections.shuffle(li);
        tisk();
      }
    },

    new Akce("hledani v zamichanem") {
      void akce() {
        int vel = li.size();
        for (int i = 0;  i < vel;  i++) {
          li.indexOf(poleVzoru[i]);
        }
      }
    }
  };

  void testy() {
    System.out.println(typListu);
    for (int i = 0;  i < poleAkci.length;  i++) {
      long zac = System.currentTimeMillis();
      poleAkci[i].akce();
      long kon = System.currentTimeMillis();
      System.out.println(poleAkci[i].typ + ": " + (kon - zac));
    }
    System.out.println();
    li.clear();
    li = null;
    System.gc();
  }
}

public class RychlostList {
  public static void main(String[] args) {
    for (int i = 1;  i <= TypyTestu.POCET_RADU;  i++) {
      System.out.println("Velikost pole: " + TypyTestu.pocetPrvku);
      TypyTestu.poleVzoru = new Integer[TypyTestu.pocetPrvku];
      for (int j = 0; j < TypyTestu.pocetPrvku; j++) {
        TypyTestu.poleVzoru[j] = Integer.valueOf(j);
      }
      new TypyTestu(new ArrayList<>()).testy();
//      new TypyTestu(new ArrayList<>(TypyTestu.pocetPrvku)).testy();
      new TypyTestu(new LinkedList<>()).testy();
      TypyTestu.pocetPrvku *= 10;
      TypyTestu.poleVzoru = null;
      System.gc();
    }
  }
}
