package kolekce;
import java.util.*;
import java.text.*;

public class OsobaProKolekce implements Comparable<OsobaProKolekce> {
  // zakladni stavove atributy
  // nemenitelne
  private final String  krestni, prijmeni;
  // menitelne
  private int vyska;
  private double vaha;
  // odvozeny atribut - bitovy obraz vahy pro hashCode() a equals()
  protected long longVaha;
  // pomocny atribut pro ceske razeni
  private static final Collator COL = 
             Collator.getInstance(new Locale("cs", "CZ"));

  public OsobaProKolekce(String krestni, String prijmeni,
                                 int vyska, double vaha) {
    if (krestni == null  ||  prijmeni == null)
      throw new NullPointerException();
    this.krestni = krestni;
    this.prijmeni = prijmeni;
    setVyska(vyska);
    setVaha(vaha);
  }

  public String getKrestni() {
    return krestni;
  }

  public String getPrijmeni() {
    return prijmeni;
  }

  public void setVyska(int vyska) {
    if (vyska <= 0)
      throw new IllegalArgumentException("vyska=" +
                                                   vyska);
    this.vyska = vyska;
  }

  public void setVaha(double vaha) {
    if (vaha <= 0)
      throw new IllegalArgumentException("vaha=" + vaha);
    this.vaha = vaha;
    this.longVaha = Double.doubleToLongBits(this.vaha);
  }

  public int getVyska() {
    return vyska;
  }

  public double getVaha() {
    return vaha;
  }

  public String toString() {
    return prijmeni + " " + krestni + ", " 
                             + vyska + ", " + vaha + "\n";
  }

  public boolean equals(Object o) {
    if (o == this)
      return true;
    if ((o instanceof OsobaProKolekce) == false)
      return false;
    OsobaProKolekce opk = (OsobaProKolekce) o;
    boolean stejneKrestni = krestni.equals(opk.krestni);
    boolean stejnePrijmeni = prijmeni.equals(opk.prijmeni);
    boolean stejnaVyska = vyska == opk.vyska;
    boolean stejnaVaha = longVaha == opk.longVaha;

    return  stejneKrestni && stejnePrijmeni 
            && stejnaVyska && stejnaVaha;
  }

  public int hashCode() {
    int vysledek = 17;
    vysledek = 37 * vysledek + krestni.hashCode();
    vysledek = 37 * vysledek + prijmeni.hashCode();
    vysledek = 37 * vysledek + vyska;
    int pom = (int) (longVaha ^ (longVaha >>> 32));
    vysledek = 37 * vysledek + pom;
    return vysledek;
  }

  // prirozene razeni
  public int compareTo(OsobaProKolekce opk) {
    int tmpP = COL.compare(this.prijmeni, opk.prijmeni);
    int tmpK = COL.compare(this.krestni, opk.krestni);
    return (tmpP == 0 ? tmpK : tmpP);
  }

  // komparatory pro absolutni razeni
  public static final Comparator<OsobaProKolekce> PODLE_VYSKY = 
                             new Comparator<OsobaProKolekce>() {
    public int compare(OsobaProKolekce o1, OsobaProKolekce o2) {
      return o1.vyska - o2.vyska;
    }
  };

  public static final Comparator<OsobaProKolekce> PODLE_VAHY = 
                                new Comparator<OsobaProKolekce>() {
    public int compare(OsobaProKolekce o1, OsobaProKolekce o2) {
      double v1 = o1.vaha;
      double v2 = o2.vaha;
      long  lv1 = o1.longVaha;
      long  lv2 = o2.longVaha;
      if (lv1 == lv2)
        return 0;
      if (v1 > v2)
        return +1;
      else
        return -1;
    }
  };
}

