package typy;

import java.util.HashSet;
import java.util.Random;
import java.util.Set;

class Osoba {
  // zakladni stavove atributy
  protected boolean muz;
  protected int vyska;
  protected double vaha;
  protected String jmeno;
  // bitovy obraz vahy pro hashCode() a equals()
  protected long longVaha; // odvozeny atribut
  private static Random r = new Random(1);

  Osoba() {
    this.muz = r.nextBoolean();
    this.vyska = 170 + r.nextInt(31);     // <170; 200>
    this.vaha = 50 + 50 * r.nextDouble(); // <50; 100)
    this.longVaha = Double.doubleToLongBits(this.vaha);
    byte[] b = new byte[5];
    for (int i = 0;  i < 5;  i++) {
      b[i] = (byte) ((r.nextInt(26) + (byte) 'a'));
    }
    jmeno = new String(b);
  }

  public String toString() {
    return jmeno + ", " + (muz ? "muz " : "zena") 
                          + ", " + vyska + ", " + vaha;
  }

  @Override
  public boolean equals(Object o) {
    if (o == this)
      return true;
    if (o instanceof Osoba == false)
      return false;
    Osoba os = (Osoba) o;
//    if (this.muz != os.muz)
//        return false;
    
    boolean bMuz = this.muz == os.muz;
    boolean bVyska = this.vyska == os.vyska;
    boolean bVaha = this.longVaha == os.longVaha;
    boolean bJmeno = this.jmeno.equals(os.jmeno);
    return bMuz && bVyska && bVaha && bJmeno;
  }
}

class NevhodnaOsoba extends Osoba {
  @Override
  public int hashCode() {
    return vyska;
  }
}

class PrijatelnaOsoba extends Osoba {
  @Override
  public int hashCode() {
    return (int) (vyska * vaha);
  }
}

class PerfektniOsoba extends Osoba {
  @Override
  public int hashCode() {
    int vysledek = 17;
    int pom;
    pom = this.muz ? 0 : 1;
    vysledek = 37 * vysledek + pom;
    pom = this.vyska;
    vysledek = 37 * vysledek + pom;
    long l = Double.doubleToLongBits(this.vaha);
    pom = (int) (l ^ (l >>> 32));
    vysledek = 37 * vysledek + pom;
    pom = this.jmeno.hashCode();
    vysledek = 37 * vysledek + pom;
    return vysledek;
  }
}

class PerfektniOsobaIdea extends Osoba {
  @Override
  public int hashCode() {
    int result;
    long temp;
    result = muz ? 1 : 0;
    result = 31 * result + vyska;
    temp = Double.doubleToLongBits(vaha);
    result = 31 * result + (int) (temp ^ temp >>> 32);
    result = 31 * result + (jmeno != null ? jmeno.hashCode() : 0);
    return result;
  }
}


final class NemennaPerfektniOsoba extends PerfektniOsoba {
//  protected int hashKod;
  private int hashKod;

  NemennaPerfektniOsoba() {
    super();
    hashKod = super.hashCode();
  }

  @Override
  public int hashCode() {
    return hashKod;
  }
}

public class TypyHashCode {
  static int pocet = 1_000_000;

  public static void main(String[] args) {
    if (args.length > 0)
      pocet = Integer.parseInt(args[0]);

    Osoba[] pole = new Osoba[pocet];

    for (int i = 0;  i < pocet;  i++) {
//      pole[i] = new NevhodnaOsoba();
//      pole[i] = new PrijatelnaOsoba();
//      pole[i] = new PerfektniOsoba();
      pole[i] = new PerfektniOsobaIdea();
//      pole[i] = new NemennaPerfektniOsoba();
    }
    
    Set<Integer> rozdilne = new HashSet<>(pocet);
    for (int i = 0;  i < pocet;  i++) {
      rozdilne.add(pole[i].hashCode());
    }
    System.out.println("Pocet rozdilnych hashCode: " + rozdilne.size());
    System.out.println("Pocet shodnych: " + (pocet - rozdilne.size()));

    System.out.println(pole[0].getClass().getName());
    long zac = System.nanoTime();
    Set<Osoba> mnOsob = new HashSet<>(pocet);
    for (int i = 0;  i < pocet;  i++) {
      mnOsob.add(pole[i]);
    }
    long kon = System.nanoTime();
    System.out.print("Vlozeni: " + mnOsob.size() + 
                                  " (" + pocet + ") ");
    System.out.println("cas = " + (kon - zac) / 1_000_000);

    zac = System.nanoTime();
    int n = 0;
    for (int i = pocet - 1;  i >= 0;  i--) {
      if (mnOsob.contains(pole[i]) == true) {
        n++;
      }
    }
    
    kon = System.nanoTime();
    System.out.print("Pristup: "+n+" (" + pocet + ") ");
    System.out.println("cas = " + (kon - zac) / 1_000_000);
  }
}
