package anotace;

import org.junit.Test;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.*;

public class JUnitTestySpusteniOsobaLepsi {

  private static Collection<Method> serazeniMetodPodlePoradi(Method[] metody) {
    int posledni = metody.length;
    Map<Integer, Method> serazeno = new TreeMap<>();
    
    for (Method metoda : metody) {
      if (metoda.isAnnotationPresent(PodminkySpusteniTestu.class)) {
        int poradi = metoda.getAnnotation(PodminkySpusteniTestu.class).poradi();
        if (poradi == 0) {
          poradi = ++posledni;
        }
        serazeno.put(poradi, metoda);
      }
    }
    return serazeno.values();
  }
  
  private static void spusteniTestu(Collection<Method> metody, MistoSpousteni misto,
                                    Object instance) {
    System.out.println("Spoustim testy na: " + misto.name());
    int proslo = 0;
    int chybne = 0;
    for (Method met : metody) {
      if (met.isAnnotationPresent(Test.class) == false) {
        continue;
      }
      if (met.isAnnotationPresent(PodminkySpusteniTestu.class) == false) {
        continue;
      }
      PodminkySpusteniTestu pst = met.getAnnotation(PodminkySpusteniTestu.class);
      Set<MistoSpousteni> mistoSet = new HashSet<>(Arrays.asList(pst.misto()));
      if (mistoSet.contains(misto) == true) {
        System.out.print(" - Test metody " + met.getName() + "() ");
        try {
          met.invoke(instance);
          System.out.println("OK");
          proslo++;
        } catch (Exception ex) {
          System.out.println("selhal z duvodu\n\t" + ex.getCause());
          chybne++;
        }
      }
    }
    System.out.println("Proslo: " + proslo + ", selhalo: " + chybne);
  }
  
  public static void main(String[] args) throws Exception {
    Class<?> zkoumanaClass = Class.forName("anotace.OsobaTest");

    Collection<Method> serazeneMetody =
        serazeniMetodPodlePoradi(zkoumanaClass.getDeclaredMethods());

    Constructor<?> konstruktor = zkoumanaClass.getDeclaredConstructor();
    Object instance = konstruktor.newInstance();
    spusteniTestu(serazeneMetody, MistoSpousteni.DESKTOP, instance);
    spusteniTestu(serazeneMetody, MistoSpousteni.VALIDATOR, instance);
  }

}
